//
//  VerjaardagenTableViewController.swift
//  BirthdayTracker
//
//  Created by Gloria Winquist on 4/10/17.
//  Copyright © 2017 iOS Kids. All rights reserved.
//

import UIKit
import CoreData
import UserNotifications

class VerjaardagenTableViewController: UITableViewController {
    
    var verjaardagen = [Verjaardag]()
    
    let dateFormatter = DateFormatter()

    override func viewDidLoad() {
        super.viewDidLoad()
        
        dateFormatter.dateStyle = .full
        dateFormatter.timeStyle = .none
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        let appDelegate = UIApplication.shared.delegate as! AppDelegate
        let context = appDelegate.persistentContainer.viewContext
        
        let fetchRequest = Verjaardag.fetchRequest() as NSFetchRequest<Verjaardag>
        
        let sortDescriptor1 = NSSortDescriptor(key: "achternaam", ascending: true)
        let sortDescriptor2 = NSSortDescriptor(key: "voornaam", ascending: true)
        fetchRequest.sortDescriptors = [sortDescriptor1, sortDescriptor2]
        
        do {
            verjaardagen = try context.fetch(fetchRequest)
        } catch let error {
                print("Kan het niet ophalen door de foutmelding: \(error).")
        }
        
        tableView.reloadData()
    }


    // MARK: - Table view data source

    override func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return verjaardagen.count
    }

    
    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        
        let cell = tableView.dequeueReusableCell(withIdentifier: "verjaardagCellIdentifier", for: indexPath)
        let verjaardag = verjaardagen[indexPath.row]
        
        let voornaam = verjaardag.voornaam ?? ""
        let achternaam = verjaardag.achternaam ?? ""
        cell.textLabel?.text = voornaam + " " + achternaam
        
        if let date = verjaardag.geboortedatum as Date? {
            cell.detailTextLabel?.text = dateFormatter.string(from: date)
        } else {
            cell.detailTextLabel?.text = ""
        }

        return cell
    }
    
    // Override to support conditional editing of the table view.
    override func tableView(_ tableView: UITableView, canEditRowAt indexPath: IndexPath) -> Bool {
        // Return false if you do not want the specified item to be editable.
        return true
    }
    
    // Override to support editing the table view.
    override func tableView(_ tableView: UITableView, commit editingStyle: UITableViewCellEditingStyle, forRowAt indexPath: IndexPath) {
        
        if verjaardagen.count > indexPath.row {
            
            let verjaardag = verjaardagen[indexPath.row]
            
            //Verwijder notificatie
            if let identifier = verjaardag.verjaardagId {
                let center = UNUserNotificationCenter.current()
                center.removePendingNotificationRequests(withIdentifiers: [identifier])
            }
            let appDelegate = UIApplication.shared.delegate as! AppDelegate
            let context = appDelegate.persistentContainer.viewContext
            
            context.delete(verjaardag)
            verjaardagen.remove(at: indexPath.row)
            
            do {
                try context.save()
            } catch let error {
                print("Kan niet opslaan door \(error).")
            }
            
            tableView.deleteRows(at: [indexPath], with: .fade)
        }
    }
    


}
