//
//  GameScene.swift
//  SchoolhouseSkateboarder
//
//  Created by MATTHEW MCCARTHY on 4/5/17.
//  Copyright © 2017 iOS Kids. All rights reserved.
//

import SpriteKit

class GameScene: SKScene {
    
    // Een array die alle huidige stoeptegels bevat
    var stenen = [SKSpriteNode]()
    
    // Het formaat van de stoeptegel graphics is gebruikt
    var steenSize = CGSize.zero
    
    // Instelling voor hoe snel het spel naar rechts scrolt
    // Dit kan toenemen als de gebruiker vordert in het spel
    var scrollSnelheid: CGFloat = 5.0
    
    // Een constante voor zwaartekracht of hoe snel objecten op de aarde vallen
    let zwaartekrachtSnelheid: CGFloat = 1.5
    
    // Het tijdstip van de laatste update
    var laatsteUpdateTijdstip: TimeInterval?
    
    // De held van het spel, de skater, is hier gemaakt
    let skater = Skater(imageNamed: "skater")
    
    override func didMove(to view: SKView) {
        
        anchorPoint = CGPoint.zero
        
        let achtergrond = SKSpriteNode(imageNamed: "achtergrond")
        let xMid = frame.midX
        let yMid = frame.midY
        achtergrond.position = CGPoint(x: xMid, y: yMid)
        addChild(achtergrond)
        
        // De skater instellen en toevoegen aan de scene
        resetSkater()
        addChild(skater)
        
        // Voeg een tik gesture recognizer toe om te weten wanneer de gebruiker op het scherm heeft getikt
        let tikMethode = #selector(GameScene.behandelTik(tikGesture:))
        let tikGesture = UITapGestureRecognizer(target: self, action: tikMethode)
        view.addGestureRecognizer(tikGesture)
    }
    
    func resetSkater() {
        
        // Stel de startpositie, zPosition en minimumY van de skater in
        let skaterX = frame.midX / 2.0
        let skaterY = skater.frame.height / 2.0 + 64.0
        skater.position = CGPoint(x: skaterX, y: skaterY)
        skater.zPosition = 10
        skater.minimumY = skaterY
    }
    
    func spawnSteen(opPositie position: CGPoint) -> SKSpriteNode {
        
        // Maak een steen sprite en voeg het aan de scene toe
        let steen = SKSpriteNode(imageNamed: "stoep")
        steen.position = position
        steen.zPosition = 8
        addChild(steen)
        
        // Update onze steenSize door de echte steen size
        steenSize = steen.size
        
        // Voeg de nieuwe steen toe aan het array van stenen
        stenen.append(steen)
        
        // Retourneer deze steen naar de aanroeper
        return steen
    }
    
    func updateStenen(metScrollhoeveelheid huidigeScrollhoeveelheid: CGFloat) {
        
        // Houd de grootste x-positie van alle huidige stenen bij
        var versteRechtersteenX: CGFloat = 0.0
        
        for steen in stenen {
            
            let nieuweX = steen.position.x - huidigeScrollhoeveelheid
            
            // Als een steen te ver naar links is verschoven (uit het scherm), verwijder hem dan
            if nieuweX < -steenSize.width {
                
                steen.removeFromParent()
                
                if let steenIndex = stenen.index(of: steen) {
                    stenen.remove(at: steenIndex)
                }
                
            } else {
                
                // Update de positie van de steen die nog op het scherm staat
                steen.position = CGPoint(x: nieuweX, y: steen.position.y)
                
                // Update de meest rechtse positietracker
                if steen.position.x > versteRechtersteenX {
                    versteRechtersteenX = steen.position.x
                }
            }
        }
        
        // Een while loop om ervoor te zorgen dat ons scherm altijd stenen bevat
        while versteRechtersteenX < frame.width {
            
            var steenX = versteRechtersteenX + steenSize.width + 1.0
            let steenY = steenSize.height / 2.0
            
            // Plaats op willekeurige plekken gaten in de stoep, zodat de speler er overheen moet springen
            let willekeurigNummer = arc4random_uniform(99)
            
            if willekeurigNummer < 5 {
                
                // Er is 5 procent kans dat er een gat gemaakt wordt tussen de stenen
                let gat = 20.0 * scrollSnelheid
                steenX += gat
            }
            
            // Laat een nieuwe steen verschijnen en update de meest rechtse steen
            let nieuweSteen = spawnSteen(opPositie: CGPoint(x: steenX, y: steenY))
            versteRechtersteenX = nieuweSteen.position.x
        }
    }
    
    func updateSkater() {
        
        if !skater.isOpGrond {
            
            // Stel de nieuwe snelheid van de skater in als het beinvloed wordt door 'zwaartekracht'
            let snelheidY = skater.snelheid.y - zwaartekrachtSnelheid
            skater.snelheid = CGPoint(x: skater.snelheid.x, y: snelheidY)
            
            // Stel de nieuwe y-positie van de skater in, gebaseerd op haar snelheid
            let nieuweSkaterY: CGFloat = skater.position.y + skater.snelheid.y
            skater.position = CGPoint(x: skater.position.x, y: nieuweSkaterY)
            
            // Controleer of de skater is geland
            if skater.position.y < skater.minimumY {
                
                skater.position.y = skater.minimumY
                skater.snelheid = CGPoint.zero
                skater.isOpGrond = true
            }
        }
    }
    
    override func update(_ currentTime: TimeInterval) {
        
        // Bepaal de verstreken tijd sinds de laatste update is aangeroepen
        var verstrekenTijd: TimeInterval = 0.0
        
        if let laatsteTijdstip = laatsteUpdateTijdstip {
            verstrekenTijd = currentTime - laatsteTijdstip
        }
        
        laatsteUpdateTijdstip = currentTime
        
        let verwachteVerstrekenTijd: TimeInterval = 1.0 / 60.0
        
        // Hier berekenen we hoe ver alles moet bewegen in deze update
        let scrollAanpassing = CGFloat(verstrekenTijd / verwachteVerstrekenTijd)
        let huidigeScrollhoeveelheid = scrollSnelheid * scrollAanpassing
        
        updateStenen(metScrollhoeveelheid: huidigeScrollhoeveelheid)
        
        updateSkater()
    }
    
    func behandelTik(tikGesture: UITapGestureRecognizer) {
        
        // Laat de skater springen als de speler op het scherm tikt wanneer de skater zich op de grond bevindt
        if skater.isOpGrond {
            
            // Stel de y-snelheid van de skater in op de aanvankelijke springsnelheid van de skater
            skater.snelheid = CGPoint(x: 0.0, y: skater.springTempo)
            
            // Houd bij dat de skater zich niet op de grond bevindt
            skater.isOpGrond = false
        }
    }
}
